@@ -281,7 +281,7 @@ xQTLanalyze_getTraits <- function(sentinelSnpDF, detectRange=1e6, tissueSiteDeta
 #' @param population Supported population is consistent with the LDlink, which can be listed using function "LDlinkR::list_pop()"
 #' @param gwasSampleNum Sample number of GWAS dataset. Default:50000.
 #' @param token LDlink provided user token, default = NULL, register for token at https://ldlink.nci.nih.gov/?tab=apiaccess
-#' @param method (character) options: "coloc"(default) or "hyprcoloc". Package `coloc` or `hyprcoloc` is required.
+#' @param method (character) options: "coloc"(default) or "hyprcoloc" (need a highe version).
 #' @param bb.alg For `hyprcoloc`, branch and bound algorithm: TRUE, employ BB algorithm; FALSE, do not. Default: FALSE.
 #'
 #' @return A list of coloc result and details.
@@ -291,8 +291,7 @@ xQTLanalyze_getTraits <- function(sentinelSnpDF, detectRange=1e6, tissueSiteDeta
 #' \donttest{
 #' url1 <- "http://raw.githubusercontent.com/dingruofan/exampleData/master/gwasDFsub_MMP7.txt"
 #' gwasDF <- data.table::fread(url1)
-#' output <- xQTLanalyze_coloc(gwasDF = gwasDF, traitGene= "MMP7", method="Both",
-#'                             tissueSiteDetail="Prostate")
+#' output <- xQTLanalyze_coloc(gwasDF = gwasDF, traitGene= "MMP7", tissueSiteDetail="Prostate")
 #' }
 xQTLanalyze_coloc <- function(gwasDF, traitGene, geneType="auto", genomeVersion="grch38", tissueSiteDetail="", study="gtex_v8", mafThreshold=0.01, population="EUR", gwasSampleNum=50000, method="coloc", token="9246d2db7917", bb.alg=FALSE){
   rsid <- chr <- position <- se <- pValue <- snpId <- maf <- pos <- i <- variantId <- se.eqtl <- se.gwas <-SNP.PP.H4 <- beta.eqtl <- beta.gwas <- posterior_prob <- regional_prob <-candidate_snp <- posterior_explained_by_snp  <- NULL
@@ -303,22 +302,14 @@ xQTLanalyze_coloc <- function(gwasDF, traitGene, geneType="auto", genomeVersion=
   # genomeVersion="grch37"
   # gwasSampleNum=50000
   # mafThreshold=0.01
+  # https://stackoverflow.com/questions/66849936/make-cran-r-package-suggest-github-r-package
   population <- ""
   token <- ""
 
-  if(method == "method" && !requireNamespace("coloc")){
+  if(method == "coloc" && !requireNamespace("coloc")){
     stop("please install package \"coloc\" with install.packages(\"coloc\").")
   }
 
-  if(method == "hyprcoloc" && !requireNamespace("hyprcoloc")){
-    stop("please install package \"hyprcoloc\" with devtools::install_github(\"cnfoley/hyprcoloc\").")
-  }
-
-  if(method == "Both"){
-    if( !requireNamespace("coloc") ){ stop("please install package \"coloc\" with install.packages(\"coloc\").") }
-    if( !requireNamespace("hyprcoloc") ){ stop("please install package \"hyprcoloc\" with devtools::install_github(\"cnfoley/hyprcoloc\").") }
-  }
-
   # Automatically determine the type of variable:
   if(geneType=="auto"){
     if( all(unlist(lapply(traitGene, function(g){ stringr::str_detect(g, "^ENSG") }))) ){
@@ -421,7 +412,7 @@ xQTLanalyze_coloc <- function(gwasDF, traitGene, geneType="auto", genomeVersion=
     message("== Using method: coloc")
     # 防止 check_dataset 中 p = pnorm(-abs(d$beta/sqrt(d$varbeta))) * 2 出错
     suppressWarnings(coloc_Out <- coloc::coloc.abf(dataset1 = list( pvalues = gwasEqtlInfo$pValue.gwas, type="quant", N=gwasSampleNum, snp=gwasEqtlInfo$rsid, MAF=gwasEqtlInfo$maf.gwas),
-                                                   dataset2 = list( pvalues = gwasEqtlInfo$pValue.eqtl, type="quant", N=sampleNum[tissueSiteDetailId, on="tissueSiteDetailId"]$sampleNum, snp=gwasEqtlInfo$rsid, MAF= gwasEqtlInfo$maf.eqtl)))
+                                                   dataset2 = list( pvalues = gwasEqtlInfo$pValue.eqtl, type="quant", N=ifelse(is.na(sampleNum[tissueSiteDetailId, on="tissueSiteDetailId"]$sampleNum), 300, sampleNum[tissueSiteDetailId, on="tissueSiteDetailId"]$sampleNum), snp=gwasEqtlInfo$rsid, MAF= gwasEqtlInfo$maf.eqtl)))
     coloc_Out_results <- as.data.table(coloc_Out$results)
     # coloc_Out_results$gene <- traitGenes[i]
     coloc_Out_summary <- as.data.table(t(as.data.frame(coloc_Out$summary)))
@@ -434,49 +425,6 @@ xQTLanalyze_coloc <- function(gwasDF, traitGene, geneType="auto", genomeVersion=
     # coloc_Out_summary$pearsonCoor <- cor(-log(gwasEqtlInfo$pValue.gwas, 10),-log(gwasEqtlInfo$pValue.eqtl, 10), method = "pearson")
 
     return(list(coloc_Out_summary=coloc_Out_summary, gwasEqtlInfo=gwasEqtlInfo))
-  }else if( method=="hyprcoloc" ){
-    message("== Using method: hyprcoloc")
-    # construct beta matrix and se matrix:
-    betasMat <- as.matrix(gwasEqtlInfo[,.( eqtl=beta.eqtl, gwas=beta.gwas)])
-    rownames(betasMat) <- gwasEqtlInfo$rsid
-    sesMat <- as.matrix(gwasEqtlInfo[,.(eqtl=se.eqtl, gwas=se.gwas)])
-    rownames(sesMat) <- gwasEqtlInfo$rsid
-    res <- hyprcoloc::hyprcoloc(effect.est = betasMat, effect.se= sesMat, trait.names=colnames(betasMat), snp.id=rownames(betasMat), bb.alg=bb.alg);
-    hyprcoloc_Out_summary <- as.data.table(res$results)
-    hyprcoloc_Out_summary$traitGene <- traitGene
-    hyprcoloc_Out_summary <- hyprcoloc_Out_summary[,.(traitGene, posterior_prob, regional_prob, candidate_snp, posterior_explained_by_snp)]
-    message(hyprcoloc_Out_summary)
-    message("== Done")
-
-    return(list(hyprcoloc_Out_summary=hyprcoloc_Out_summary, gwasEqtlInfo=gwasEqtlInfo))
-  }else if( method=="Both"){
-    message("== Using methods: coloc AND hyprcoloc.")
-    # coloc:
-    suppressWarnings(coloc_Out <- coloc::coloc.abf(dataset1 = list( pvalues = gwasEqtlInfo$pValue.gwas, type="quant", N=gwasSampleNum, snp=gwasEqtlInfo$rsid, MAF=gwasEqtlInfo$maf.gwas),
-                                                   dataset2 = list( pvalues = gwasEqtlInfo$pValue.eqtl, type="quant", N=sampleNum[tissueSiteDetailId, on="tissueSiteDetailId"]$sampleNum, snp=gwasEqtlInfo$rsid, MAF= gwasEqtlInfo$maf.eqtl)))
-    coloc_Out_results <- as.data.table(coloc_Out$results)
-    # coloc_Out_results$gene <- traitGenes[i]
-    coloc_Out_summary <- as.data.table(t(as.data.frame(coloc_Out$summary)))
-    coloc_Out_summary$traitGene <- traitGene
-    coloc_Out_summary$candidate_snp <- coloc_Out_results[order(-SNP.PP.H4)][1,]$snp
-    coloc_Out_summary$SNP.PP.H4 <- coloc_Out_results[order(-SNP.PP.H4)][1,]$SNP.PP.H4
-    # print(coloc_Out_summary)
-
-    message("")
-    # hyprcoloc:
-    betasMat <- as.matrix(gwasEqtlInfo[,.( eqtl=beta.eqtl, gwas=beta.gwas)])
-    rownames(betasMat) <- gwasEqtlInfo$rsid
-    sesMat <- as.matrix(gwasEqtlInfo[,.(eqtl=se.eqtl, gwas=se.gwas)])
-    rownames(sesMat) <- gwasEqtlInfo$rsid
-    res <- hyprcoloc::hyprcoloc(effect.est = betasMat, effect.se= sesMat, trait.names=colnames(betasMat), snp.id=rownames(betasMat), bb.alg=FALSE);
-    hyprcoloc_Out_summary <- as.data.table(res$results)
-    hyprcoloc_Out_summary$traitGene <- traitGene
-    hyprcoloc_Out_summary <- hyprcoloc_Out_summary[,.(traitGene, hypr_posterior=posterior_prob, hypr_regional_prob=regional_prob, hypr_candidate_snp=candidate_snp, hypr_posterior_explainedBySnp=posterior_explained_by_snp)]
-    colocOut <- merge(coloc_Out_summary, hyprcoloc_Out_summary, by="traitGene" )
-    print(colocOut)
-
-    message("== Done")
-    return(list(colocOut=colocOut, gwasEqtlInfo=gwasEqtlInfo))
   }
 }
 
@@ -560,8 +508,8 @@ xQTLanalyze_TSExp <- function(genes, geneType="auto", method="SPM", datasetId="g
 #' @param geneType (character) options: "auto","geneSymbol" or "gencodeId". Default: "auto".
 #' @param variantName (character) name of variant, dbsnp ID and variant id is supported, eg. "rs138420351" and "chr17_7796745_C_T_b38".
 #' @param variantType (character) options: "auto", "snpId" or "variantId". Default: "auto".
-#' @param binNum A integer value. Number of bins to split values of R2 of LD into homogeneous bins.
-#' @param study Studies can be listed using "ebi_study_tissues". If the study is null, use all studies (Default).
+#' @param tissueLabels (a character vector) can be listed with `ebi_study_tissues`. If is null, use all tissue / cell-types. (Default)
+#' @param study (character) Studies can be listed using `ebi_study_tissues`. If is null, use all studies (Default).
 #' @param population (string) One of the 5 popuations from 1000 Genomes: 'AFR', 'AMR', 'EAS', 'EUR', and 'SAS'.
 #' @import data.table
 #' @import stringr
@@ -570,15 +518,41 @@ xQTLanalyze_TSExp <- function(genes, geneType="auto", method="SPM", datasetId="g
 #'
 #' @examples
 #' \donttest{
-#' speDT <- xQTLanalyze_qtlSpecificity(gene="MMP7", variantName="rs11568818", study="")
-#' xQTLvisual_qtlSpecificity(speDT, outPlot = "heatmap")
-#' xQTLvisual_qtlSpecificity(speDT, outPlot = "regression")
+#' propensityRes <- xQTLanalyze_propensity( gene="MMP7", variantName="rs11568818", study="TwinsUK")
+#' xQTLvisual_qtlPropensity(propensityRes)
 #' }
-xQTLanalyze_qtlSpecificity <- function(gene="", geneType="auto", variantName="", variantType="auto", binNum=4, study="", population="EUR"){
+xQTLanalyze_propensity <- function(gene="", geneType="auto", variantName="", variantType="auto", tissueLabels="", study="", population="EUR"){
   .<- slope <- logP_minMax <- NULL
-  study_accession <- tissue_label <- R2 <-snpId <- pValue <- tissue <- study_id <- qtl_group <- SNP_B <- LDbins <- logP <- corRP <- NULL
+  study_accession <- tissue_label <- pos <- snpPanel <- variantId <- R2 <-snpId <- pValue <- tissue <- study_id <- qtl_group <- SNP_B <- LDbins <- logP <- corRP <- NULL
+  pValue_propensity <- pValue_eQTL <- NULL
+
+  # binNum A integer value. Number of bins to split values of R2 of LD into homogeneous bins.
+  binNum=4
+  ebi_ST <- data.table::copy(ebi_study_tissues)
+
+  # study- tissue:
+  if( all(tissueLabels!="") & study!=""){
+    ebi_ST <- ebi_ST[which(tolower(tissue_label) %in% tolower(tissueLabels) & tolower(ebi_ST$study_accession) == tolower(study) ), ][,.SD[1,], by="tissue_label"]
+    message(study)
+  }
+
+  # all tissue- study:
+  if(study=="" & all(tissueLabels=="") ){
+    ebi_ST <- ebi_ST[,.SD[1,], by="tissue_label"]
+  }
+
+  if(study!="" & all(tissueLabels=="") ){
+    ebi_ST <- ebi_ST[ which(tolower(ebi_ST$study_accession) == tolower(study)),][,.SD[1,], by="tissue_label"]
+  }
+
+  if(study=="" & all(tissueLabels!="") ){
+    ebi_ST <- ebi_ST[ which(tolower(tissue_label) %in% tolower(tissueLabels)),][,.SD[1,], by="tissue_label"]
+  }
+
+  if(nrow(ebi_ST)==0){
+    stop("Please check study id or tissue label.")
+  }
 
-  ebi_ST <-copy(ebi_study_tissues)
   if(gene=="" || variantName==""){
     stop("gene and variant can not be null!")
   }
@@ -661,7 +635,6 @@ xQTLanalyze_qtlSpecificity <- function(gene="", geneType="auto", variantName="",
     binNumForSample_tmp <- 10
     varNumForSample_tmp <- 10
     snpLD$LDbins <- as.character(cut(snpLD$R2, breaks=seq(0,1,length.out=(binNumForSample_tmp+1)) ))
-    set.seed(1)
     snpLDForSample_tmp <- as.data.frame(tapply( 1:nrow(snpLD), snpLD$LDbins, function(x){ if(length(x)<=varNumForSample_tmp){return(x)}else{ return(sample(x, varNumForSample_tmp, replace = FALSE)) } }))
     names(snpLDForSample_tmp) <- "ID"
     snpLDForSample_tmp$LDbins <- rownames(snpLDForSample_tmp)
@@ -671,21 +644,60 @@ xQTLanalyze_qtlSpecificity <- function(gene="", geneType="auto", variantName="",
     rm(binNumForSample_tmp, varNumForSample_tmp, snpLDForSample_tmp)
   }
 
-
-  message("== Start download associations of QTL...")
+  # Hypothesis testing
+  # 对于每个组织分别进行假设检验
+  # 1. 获取所有组织该基因的 eQTL. 2. 为每个 LD-associated gene构建panel. 3. 获得real panel和simulated panel 的SNP. 4.
+  ebi_ST$pValue_eQTL <- (-1)
+  ebi_ST$pValue_propensity <- (-1)
   assoAll <- data.table()
-  for(i in 1:nrow(snpLD)){
-    suppressMessages( asso_I <- xQTLdownload_eqtlAllAsso(gene = gene, variantName= snpLD[i,]$SNP_B, study=study, withB37VariantId=FALSE) )
-    if( is.null(asso_I) || nrow(asso_I)==0){
-      message("== Num:",i,"/",nrow(snpLD), ", SNP: [",snpLD[i,]$SNP_B,"], got records: ", 0,", Skipped. ", format(Sys.time(), "| %Y-%b-%d %H:%M:%S "))
+  for( i in 1:nrow(ebi_ST)){
+    # 如果有多个 qtl group:
+    message("")
+    message("==> For tissue ",ebi_ST[i,]$tissue_label, " (",i,"/",nrow(ebi_ST),")")
+    geneAsso_i <- xQTLdownload_eqtlAllAsso(gene=gene, geneType = geneType, tissueLabel = ebi_ST[i,]$tissue_label, study=ebi_ST[i,]$study_accession)
+    if(!exists("geneAsso_i") ||is.null(geneAsso_i)|| nrow(geneAsso_i)==0){
       next()
     }else{
-      message("== Num:",i,"/",nrow(snpLD), "; SNP: [",snpLD[i,]$SNP_B,"]; got records: [", nrow(asso_I),"]; Tissues: [", length(unique(asso_I$tissue_label)),"]; Studies: [", length(unique(asso_I$study_id)), format(Sys.time(), "] | %Y-%b-%d %H:%M:%S "))
+      geneAsso_i <- geneAsso_i[ qtl_group ==geneAsso_i[1,]$qtl_group,][order(pos)]
     }
-    asso_I <- asso_I[,.(snpId, pValue, beta, tissue, tissue_label, study_id, qtl_group)]
-    assoAll <- rbind(assoAll,asso_I)
+    # LD-associated gene for plot:
+    asso_I <- geneAsso_i[snpId %in% union(snpLD$SNP_B, snpLD$SNP_A),.(snpId, pValue, beta, tissue, tissue_label, study_id, qtl_group)]
+    assoAll <- rbind(assoAll, asso_I)
     rm(asso_I)
+
+    # all variants of gene:
+    assoAllLd_i <- geneAsso_i[,.(snpId, pos, pValue)]
+    assoAllLd_i <- merge(assoAllLd_i, snpLD[,.(snpId=SNP_B,R2)], by="snpId", sort=FALSE)
+    assoAllLd_i$snpPanel <- paste0("s",1:nrow(assoAllLd_i))
+
+    assoControl <- rbindlist(lapply(1:nrow(assoAllLd_i), function(snp_j){
+      tmp <- geneAsso_i[data.table(ID=1:nrow(geneAsso_i), pos = geneAsso_i$pos, diff=abs(geneAsso_i$pos-assoAllLd_i[snp_j,]$pos))[diff!=0][order(diff)][1:5,]$ID,.(snpId, pos, pValue)]
+      tmp$snpPanel <- assoAllLd_i[snp_j,]$snpPanel
+      return(tmp)
+    }))
+    #
+
+    asso_i <- rbind(assoAllLd_i[,-c("R2")], assoControl)
+    asso_i <- merge(asso_i,  assoAllLd_i[,.(snpPanel,R2)], by="snpPanel")
+    message("==> Start calculating p-value in ",ebi_ST[i,]$tissue_label, "; ",i,"/",nrow(ebi_ST))
+    corValues <- unlist(lapply(1:1000, function(x){
+      if(x %% 100 ==0){ message("==> Complete ",x/10,"% in tissue: ",  ebi_ST[i,]$tissue_label ," (",i,"/",nrow(ebi_ST),")")}
+      sampleSnps <- rbindlist(lapply(unique(asso_i$snpPanel), function(xx){ a=asso_i[snpPanel==xx];a[sample(1:nrow(a),1),] }))
+      return(cor(sampleSnps$R2, -log10(sampleSnps$pValue)))
+    }))
+    corReal <- cor(assoAllLd_i$R2, -log10(assoAllLd_i$pValue))
+    ebi_ST[i, "pValue_propensity"] <- 1-(length(which(corReal>corValues)))/1001
+    ebi_ST[i, "pValue_eQTL"] <- geneAsso_i[snpId==variantInfo$snpId,]$pValue
+    rm(geneAsso_i, assoAllLd_i, assoControl, asso_i, corValues, corReal)
   }
+  ebi_ST[pValue_propensity == (-1),"pValue_propensity"] <-NA
+  ebi_ST[pValue_eQTL == (-1),"pValue_eQTL"] <-NA
+  tissuePropensity  <- copy(ebi_ST)
+
+  if(all(is.na(tissuePropensity$pValue_propensity))){
+    stop("No associations were fetched from ", study)
+  }
+
   # Retain min pvalue in each tissue_label-study, due to the duplication induced by qtl_group.
   assoAll <- assoAll[,.SD[which.min(pValue),], by=c("tissue", "tissue_label", "study_id", "qtl_group", "snpId")]
   assoAllLd <- merge(snpLD[,.(snpId=SNP_B, R2)], assoAll, by="snpId")
@@ -693,6 +705,19 @@ xQTLanalyze_qtlSpecificity <- function(gene="", geneType="auto", variantName="",
   # scale logP by tissue group:
   assoAllLd <- assoAllLd[,.(snpId, R2, beta, logP, logP_minMax=(logP-min(logP))/(max(logP)-min(logP))),by="tissue_label"]
 
+  # saveRDS(ebi_ST,"../ebi_ST_FLOT1.rds")
+  # query eQTL in each tissue:
+  # eQTLs <- data.table()
+  # for( i in 1:nrow(ebi_ST)){
+  #   eQTL_i <- xQTLdownload_eqtlAllAsso(gene=gene, variantName = variantName, tissueLabel = ebi_ST[i,]$tissue_label, study=ebi_ST[i,]$study_id)
+  #   if(!exists("eQTL_i") ||is.null(eQTL_i)|| nrow(eQTL_i)==0){
+  #     next()
+  #   }else{
+  #     eQTL_i <- eQTL_i[,.(variantId, snpId, pos, tissue_label, study_id, pValue)]
+  #   }
+  #   eQTLs <- rbind(eQTLs, eQTL_i)
+  # }
+  # tissueSpecificity <- merge(ebi_ST, eQTLs[,.(tissue_label, study_id, pValue)], by=c("tissue_label", "study_id"))
 
   # lm:
   lm_f <- function(DT){
@@ -701,9 +726,10 @@ xQTLanalyze_qtlSpecificity <- function(gene="", geneType="auto", variantName="",
   lm_R2_logP <- assoAllLd[,lm_f(.SD), by="tissue_label"][order(slope)]
 
   # cor:
-  cor_R2_logP <- assoAllLd[,.(corRP=cor(R2, logP_minMax)),by="tissue_label"][order(corRP)]
+  cor_R2_logP <- assoAllLd[,.(corRP=cor(R2, logP_minMax), corPvalue=cor.test(R2, logP_minMax)$p.value),by="tissue_label"][order(corRP)]
+  cor_R2_logP$logCorP <- log10( cor_R2_logP$corPvalue)*(-1)
 
-  return(list(snpLD=snpLD, assoAllLd=assoAllLd, lm_R2_logP=lm_R2_logP, cor_R2_logP= cor_R2_logP))
+  return(list(snpLD=snpLD, tissuePropensity=tissuePropensity, cor_R2_logP=cor_R2_logP, lm_R2_logP=lm_R2_logP, assoAllLd=assoAllLd))
 }
 
 
