@@ -11,7 +11,7 @@ knitr::opts_chunk$set(
 #  library(stringr)
 
 ## ---- results = 'hide', echo=TRUE, message=FALSE, warning=FALSE, eval=FALSE----
-#  speDT <- xQTLanalyze_qtlSpecificity(gene="MMP7", variantName="rs11568818", study="")
+#  propensityRes <- xQTLanalyze_propensity(gene="MMP7", variantName="rs11568818", study="")
 
 ## ---- message=FALSE, warning=FALSE, include=FALSE-----------------------------
 temp1 <- tempfile(fileext=".zip")
@@ -21,17 +21,11 @@ close(file(temp1))
 rm(temp1)
 
 ## ---- message=FALSE, warning=FALSE, eval=TRUE---------------------------------
-names(speDT)
+names(propensityRes)
 
 ## ---- results = 'hide', echo=TRUE, message=FALSE, warning=FALSE, eval=FALSE----
-#  xQTLvisual_qtlSpecificity(speDT, outPlot = "heatmap")
+#  xQTLvisual_qtlPropensity(propensityRes)
 
 ## ---- heatmap_MMP7.png, fig.align = 'center', out.width = "65%", echo=FALSE----
 # knitr::include_graphics("images/eqtl_specificity/heatmap_MMP7.png")
 
-## ---- results = 'hide', echo=TRUE, message=FALSE, warning=FALSE, eval=FALSE----
-#  xQTLvisual_qtlSpecificity(speDT, outPlot = "regression")
-
-## ---- regression_MMP7.png, fig.align = 'center', out.width = "65%", echo=FALSE----
-# knitr::include_graphics("images/eqtl_specificity/regression_MMP7.png")
-
